#include "cppm_out.h"

// Reads up to six input channels from any normal receiver of any make.
// This is convenient when working cross-brand (e.g. Futaba to Spektrum) because you
// can swap the order of the channels by flipping around the wires connected to the receiver
// outputs a 'buddy box' signal that works with most radios - Futaba, JR, Spektrum, etc.
// As uploaded here, the buddy signal is the standard negative going pulses, 400us wide
// with a 22.5ms frame.  You can easily alter the output pin from D0 to D9, swap to
// positive-going pulses, or alter from an active to an 'open collector' type drive by
// commenting or uncommenting some #define statements up near the top of cppm_out.h
// ceptimus June 2020

void setup() {
  for (uint8_t i = 0; i < 6; i++)
    cppm_out::set_us(i, 1500); // default most channels to neutral

  cppm_out::set_us(0, 1080); // default (Spektrum) throttle channel to standard throttle low
  cppm_out::set_us(2, 1100); // default (Futaba) throttle channel to standard throttle low

  cppm_out::start();
  
  // pins 2-7 are the (up to) 6 input channels connected to the Rx
  for (uint8_t i = 2; i < 8; i++)
    pinMode(i, INPUT_PULLUP);
  cli();
  PCICR |= 0b00000100; // turn on port D pin change interrupts
  PCMSK2 = 0b11111100; // allow pins D2-D7 to generate interrupts
  sei();
}

uint8_t prevD = 0;
uint32_t startPulse[6];

ISR(PCINT2_vect) {
  uint32_t now = micros();
  uint8_t pinD = PIND; // snapshot of pin states
  uint8_t changed = pinD ^ prevD; // which pin(s) changed
  uint8_t mask = 0b00000100;
  for (uint8_t i = 0; i < 6; i++) {
    if (changed & mask) { // pin for channel [i] has changed
      if (pinD & mask) { // rising edge
        startPulse[i] = now;
      } else { // falling edge
        cppm_out::set_us(i, (uint16_t)(now - startPulse[i]));
      }
    }
    mask <<= 1;
  }

  prevD = pinD; // remember state for next interrupt
}

void loop() {
}
